package com.franckbarbier.From_Java_9;

import static com.franckbarbier.From_Java_9.Direction.North;

enum Direction {
    East, South, West, North
}

public class From_Java_9 {

    public static void main(String[] args) {
        // From Java 9:
        Mammutidae m0 = new Mammutidae("Macron");
        Mammutidae m1 = new Mammutidae("Poutine");
        Mammutidae m2 = new Mammutidae("Trump");
        System.out.println(m1.description()); // Inconsistency comes from failing overriding...

        // Double braces:
        java.util.Set<Integer> primes = new java.util.HashSet<>() { // <- Creation of an anonymous inner class, which extends 'java.util.HashSet'
            { // <- Block initializer
                add(2);
                add(3);
            }
        };
        try { // Java 10 'copyOf':
            java.util.Set.copyOf(primes).add(4);
        } catch (UnsupportedOperationException uoe) {
            System.err.println("In essence, immutable collections do not support changes like additions: " + uoe.toString());
        }

        // Immutable collections from Java 9 (e.g., a set):
        try {
            java.util.Set.of(m1, m2).add(m0); // Poutine and Trump don't want Macron!
        } catch (UnsupportedOperationException uoe) {
            System.err.println("In essence, immutable collections do not support changes like additions: " + uoe.toString());
        }

        // Stream API in Java 9:
        java.util.List<Integer> positives = java.util.Arrays.asList(0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12);
        java.util.stream.Stream<Integer> stream = positives.stream().filter(p -> p > 1 && java.util.stream.IntStream.rangeClosed(2, (int) Math.sqrt(p)).noneMatch(i -> (p % i == 0)));
        System.out.println(stream.takeWhile(p -> !p.equals(7)).collect(java.util.stream.Collectors.toList())); // '[2, 3, 5]' is displayed...
        java.util.stream.Stream<Integer> stream_ = positives.stream().filter(p -> p > 1 && java.util.stream.IntStream.rangeClosed(2, (int) Math.sqrt(p)).noneMatch(i -> (p % i == 0)));
        System.out.println(stream_.dropWhile(p -> !p.equals(7)).collect(java.util.stream.Collectors.toList())); // '[7, 11]' is displayed...

        java.util.List<Object> objects = java.util.Arrays.asList(new Object(), null, new Object());
        java.util.stream.Stream<Object> stream__ = objects.stream().flatMap(o -> java.util.stream.Stream.ofNullable(o));
        System.out.println(stream__.count()); // '2' is displayed...
    }
}
